@@ -2,17 +2,17 @@
 import string
 
 from prompt_toolkit.application.current import get_app
-from prompt_toolkit.completion import PathCompleter
-from prompt_toolkit.key_binding import KeyBindings
 from prompt_toolkit.keys import Keys
-from prompt_toolkit.layout import HorizontalAlign, HSplit, VSplit
+from prompt_toolkit.layout import HSplit, VSplit
 from prompt_toolkit.layout.dimension import Dimension as D
-from prompt_toolkit.widgets import Label, TextArea, Button, Box
+from prompt_toolkit.widgets import Box, Button, Label, TextArea
 
-from ..widgets import SelectMany, SelectOne, MaskedInput, DateRange
-from .base import QHandler, register
+from interrogatio.handlers.base import QHandler
+from interrogatio.handlers.registry import register
+from interrogatio.widgets import DateRange, MaskedInput, SelectMany, SelectOne
 
 
+@register('input')
 class StringHandler(QHandler):
 
     def get_widget_class(self):
@@ -21,7 +21,7 @@ class StringHandler(QHandler):
     def get_widget_init_kwargs(self):
         kwargs = dict(
             multiline=self._question.get('multiline', False),
-            style='class:input.answer'
+            style='class:input.answer',
         )
         if 'default' in self._question:
             kwargs['text'] = self._question['default']
@@ -33,7 +33,7 @@ class StringHandler(QHandler):
         vsplit_components = [widget]
         if 'message' in self._question and self._question['message']:
             vsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['message'],
                     dont_extend_width=True,
@@ -43,7 +43,7 @@ class StringHandler(QHandler):
         hsplit_components = [VSplit(vsplit_components, padding=1)]
         if 'description' in self._question and self._question['description']:
             hsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['description'],
                     style='class:input.question',
@@ -51,28 +51,11 @@ class StringHandler(QHandler):
             )
         return HSplit(hsplit_components, padding=1)
 
-
-    def get_keybindings(self):
-        bindings = KeyBindings()
-
-        @bindings.add(Keys.ControlC)
-        def _ctrl_c(event):
-            get_app().exit(result=False)
-
-        @bindings.add(Keys.Enter)
-        def _enter(event):
-            get_app().exit(result=True)
-
-        return bindings
-
     def get_value(self):
         return self.get_widget().text
 
 
-register('input', StringHandler)
-
-
-
+@register('password')
 class PasswordHandler(QHandler):
 
     def get_widget_class(self):
@@ -82,7 +65,7 @@ class PasswordHandler(QHandler):
         kwargs = dict(
             multiline=False,
             style='class:password.answer',
-            password=True
+            password=True,
         )
         if 'default' in self._question:
             kwargs['text'] = self._question['default']
@@ -92,17 +75,13 @@ class PasswordHandler(QHandler):
         return self.get_widget().text
 
     def get_layout(self):
-        msg = '{}{}'.format(
-            self._question['message'],
-            self._question.get('question_mark', ' ?')
-        )
         widget = self.get_widget()
         widget.buffer.cursor_position = len(widget.text)
 
         vsplit_components = [widget]
         if 'message' in self._question and self._question['message']:
             vsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['message'],
                     dont_extend_width=True,
@@ -112,7 +91,7 @@ class PasswordHandler(QHandler):
         hsplit_components = [VSplit(vsplit_components, padding=1)]
         if 'description' in self._question and self._question['description']:
             hsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['description'],
                     style='class:password.question',
@@ -121,22 +100,7 @@ class PasswordHandler(QHandler):
         return HSplit(hsplit_components, padding=1)
 
 
-    def get_keybindings(self):
-        bindings = KeyBindings()
-
-        @bindings.add(Keys.ControlC)
-        def _ctrl_c(event):
-            get_app().exit(exception=KeyboardInterrupt)
-
-        @bindings.add(Keys.Enter)
-        def _enter(event):
-            get_app().exit(result=self.get_answer())
-
-        return bindings
-
-register('password', PasswordHandler)
-
-
+@register('selectone')
 class SelectOneHandler(QHandler):
 
     def get_widget_class(self):
@@ -148,7 +112,7 @@ class SelectOneHandler(QHandler):
     def get_widget_init_kwargs(self):
         kwargs = dict(
             values=self._question['values'],
-            style='class:selectone.answer'
+            style='class:selectone.answer',
         )
         if 'default' in self._question:
             kwargs['default'] = self._question['default']
@@ -159,17 +123,18 @@ class SelectOneHandler(QHandler):
         vsplit_components = [self.get_widget()]
         if 'message' in self._question and self._question['message']:
             vsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['message'],
                     dont_extend_width=True,
+                    dont_extend_height=False,
                     style='class:selectone.question',
                 ),
             )
         hsplit_components = [VSplit(vsplit_components, padding=1)]
         if 'description' in self._question and self._question['description']:
             hsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['description'],
                     style='class:selectone.question',
@@ -177,39 +142,25 @@ class SelectOneHandler(QHandler):
             )
         return HSplit(hsplit_components, padding=1)
 
-
-    def get_keybindings(self):
-
-        bindings = KeyBindings()
-
-        @bindings.add(Keys.ControlC)
-        def _ctrl_c(event):
-            get_app().exit(exception=KeyboardInterrupt)
-
-        def accept_handler(value):
-            get_app().exit(result=self.get_answer())
-
-        self.get_widget().accept_handler = accept_handler
-
-        return bindings
-
-register('selectone', SelectOneHandler)
+    def get_formatted_value(self):
+        value = self.get_value()
+        options = {t[0]: t[1] for t in self._question['values']}
+        return f'{options[value]} ({value})'
 
 
+@register('selectmany')
 class SelectManyHandler(QHandler):
 
-
     def get_widget_class(self):
         return SelectMany
 
-
     def get_value(self):
         return list(self.get_widget().checked)
 
     def get_widget_init_kwargs(self):
         kwargs = dict(
             values=self._question['values'],
-            style='class:selectmany.answer'
+            style='class:selectmany.answer',
         )
         if 'checked' in self._question:
             kwargs['checked'] = set(self._question['checked'])
@@ -223,10 +174,11 @@ class SelectManyHandler(QHandler):
         vsplit_components = [widget]
         if 'message' in self._question and self._question['message']:
             vsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['message'],
                     dont_extend_width=True,
+                    dont_extend_height=False,
                     style='class:selectone.question',
                 ),
             )
@@ -242,7 +194,7 @@ class SelectManyHandler(QHandler):
         hsplit_components = [buttons, VSplit(vsplit_components, padding=1)]
         if 'description' in self._question and self._question['description']:
             hsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['description'],
                     style='class:selectone.question',
@@ -252,169 +204,36 @@ class SelectManyHandler(QHandler):
 
     def get_keybindings(self):
 
-        bindings = KeyBindings()
+        bindings = super().get_keybindings()
 
-        @bindings.add(Keys.ControlC)
-        def _ctrl_c(event):
-            get_app().exit(exception=KeyboardInterrupt)
+        @bindings.add(Keys.Tab)
+        def _tab(event):
+            get_app().layout.focus_next()
 
-        def accept_handler(value):
-            get_app().exit(result=self.get_answer())
-
-        self.get_widget().accept_handler = accept_handler
+        @bindings.add(Keys.BackTab)
+        def _s_tab(event):
+            get_app().layout.focus_previous()
 
         return bindings
 
-register('selectmany', SelectManyHandler)
+    def get_formatted_value(self):
+        values = self.get_value()
+        if not values:
+            return ''
+        options = {t[0]: t[1] for t in self._question['values']}
 
-# class TextHandler(QHandler):
+        formatted_values = []
 
-#     def get_widget_class(self):
-#         return TextArea
+        for idx, value in enumerate(values):
+            if idx > 14:
+                formatted_values.append('...')
+                break
+            formatted_values.append(f'{options[value]} ({value})')
 
-
-#     def get_widget_init_kwargs(self):
-#         kwargs = dict(
-#             multiline=True,
-#             height=4,
-#             style='class:text.answer'
-#         )
-#         if 'default' in self._question:
-#             kwargs['text'] = self._question['default']
-#         return kwargs
-
-#     def get_value(self):
-#         return self.get_widget().text
-
-#     def get_layout(self):
-#         msg = '{}{}'.format(
-#             self._question['message'],
-#             self._question.get('question_mark', ' ?')
-#         )
-#         extra_args = self.get_init_extra_args()
-#         if 'rows' in extra_args:
-#             self.get_widget().height = int(extra_args['rows'])
-#         return VSplit(
-#             [
-#                 Label(msg,
-#                       dont_extend_width=True,
-#                       style='class:text.question'),
-#                 self.get_widget()
-#             ],
-#             padding=1)
-
-#     def get_keybindings(self):
-#         bindings = KeyBindings()
-
-#         @bindings.add(Keys.ControlC)
-#         def _ctrl_c(event):
-#             get_app().exit(exception=KeyboardInterrupt)
-
-#         @bindings.add(Keys.ControlX)
-#         def _enter(event):
-#             get_app().exit(result=self.get_answer())
-
-#         return bindings
-
-
-# class PathHandler(StringHandler):
-#     def get_widget(self):
-#         widget = super(PathHandler, self).get_widget()
-#         widget.completer = PathCompleter()
-#         return widget
-
-# register('path', PathHandler)
-
-# class RePasswordHandler(QHandler):
-
-#     ALIAS = 'repassword'
-
-
-#     def __init__(self, *args, **kwargs):
-#         super(RePasswordHandler, self).__init__(*args, **kwargs)
-#         self._rewidget = TextArea(**self.get_widget_init_kwargs())
-#         self._rewidget.buffer.cursor_position = len(self._rewidget.text)
-
-
-#     def get_widget_class(self):
-#         return TextArea
-
-#     def get_widget_init_kwargs(self):
-#         kwargs = dict(
-#             multiline=False,
-#             style='class:repassword.answer',
-#             password=True
-#         )
-#         if 'default' in self._question:
-#             kwargs['text'] = self._question['default']
-#         return kwargs
-
-#     def get_value(self):
-#         return self.get_widget().text
-
-#     def get_layout(self):
-#         msg = '{}{}'.format(
-#             self._question['message'],
-#             self._question.get('question_mark', ' ?')
-#         )
-#         extra_args = self.get_init_extra_args()
-#         remsg = '{}{}'.format(
-#             extra_args.get('remessage', 'Please repeat to confirm'),
-#             self._question.get('question_mark', ' ?')
-#         )
-#         lbl_msg = Label(msg,
-#                         dont_extend_width=True,
-#                         style='class:repassword.question')
-#         lbl_remsg = Label(remsg,
-#                           dont_extend_width=True,
-#                           style='class:repassword.question')
-#         return HSplit(
-#             [
-#                 VSplit([lbl_msg, self.get_widget()], padding=1),
-#                 VSplit([lbl_remsg, self._rewidget], padding=1)
-#             ]
-#         )
-
-#     def get_keybindings(self):
-#         bindings = KeyBindings()
-
-#         @bindings.add(Keys.ControlC)
-#         def _ctrl_c(event):
-#             get_app().exit(exception=KeyboardInterrupt)
-
-
-#         @bindings.add('tab')
-#         def _tab(event):
-#             get_app().layout.focus_next()
-
-#         @bindings.add('s-tab')
-#         def _stab(event):
-#             get_app().layout.focus_previous()
-
-#         @bindings.add(Keys.Enter)
-#         def _enter(event):
-#             if get_app().layout.has_focus(self._rewidget):
-#                 get_app().exit(result=self.get_answer())
-#             else:
-#                 get_app().layout.focus_next()
-
-#         return bindings
-
-#     def is_valid(self):
-#         extra_args = self.get_init_extra_args()
-#         super(RePasswordHandler, self).is_valid()
-#         if self.get_widget().text != self._rewidget.text:
-#             self._errors.append(
-#                 extra_args.get(
-#                     'reerror',
-#                     'Password and repeat password doesn\'t match'
-#                 )
-#             )
-#         return not self.errors
-
-# register('repassword', RePasswordHandler)
+        return ', '.join(formatted_values)
 
 
+@register('maskedinput')
 class MaskedInputHandler(QHandler):
 
     def get_widget_class(self):
@@ -432,7 +251,7 @@ class MaskedInputHandler(QHandler):
         vsplit_components = [widget]
         if 'message' in self._question and self._question['message']:
             vsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['message'],
                     dont_extend_width=True,
@@ -442,7 +261,7 @@ class MaskedInputHandler(QHandler):
         hsplit_components = [VSplit(vsplit_components, padding=1)]
         if 'description' in self._question and self._question['description']:
             hsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['description'],
                     style='class:input.question',
@@ -450,27 +269,11 @@ class MaskedInputHandler(QHandler):
             )
         return HSplit(hsplit_components, padding=1)
 
-    def get_keybindings(self):
-        bindings = KeyBindings()
-
-        @bindings.add(Keys.ControlC)
-        def _ctrl_c(event):
-            get_app().exit(result=False)
-
-        @bindings.add(Keys.Enter)
-        def _enter(event):
-            get_app().exit(result=True)
-
-        return bindings
-
     def get_value(self):
         return self.get_widget().value
 
 
-register('maskedinput', MaskedInputHandler)
-
-
-
+@register('date')
 class DateHandler(QHandler):
 
     def get_widget_class(self):
@@ -489,7 +292,7 @@ class DateHandler(QHandler):
         vsplit_components = [widget]
         if 'message' in self._question and self._question['message']:
             vsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['message'],
                     dont_extend_width=True,
@@ -499,7 +302,7 @@ class DateHandler(QHandler):
         hsplit_components = [VSplit(vsplit_components, padding=1)]
         if 'description' in self._question and self._question['description']:
             hsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['description'],
                     style='class:input.question',
@@ -507,25 +310,11 @@ class DateHandler(QHandler):
             )
         return HSplit(hsplit_components, padding=1)
 
-    def get_keybindings(self):
-        bindings = KeyBindings()
-
-        @bindings.add(Keys.ControlC)
-        def _ctrl_c(event):
-            get_app().exit(result=False)
-
-        @bindings.add(Keys.Enter)
-        def _enter(event):
-            get_app().exit(result=True)
-
-        return bindings
-
     def get_value(self):
         return self.get_widget().value
 
 
-register('date', DateHandler)
-
+@register('daterange')
 class DateRangeHandler(QHandler):
 
     def get_widget_class(self):
@@ -538,7 +327,7 @@ class DateRangeHandler(QHandler):
         if 'from_label' in self._question:
             kwargs['from_label'] = self._question['from_label']
         if 'to_label' in self._question:
-            kwargs['to_label'] = self._question['to_label']     
+            kwargs['to_label'] = self._question['to_label']
         return kwargs
 
     def get_layout(self):
@@ -546,17 +335,18 @@ class DateRangeHandler(QHandler):
         vsplit_components = [widget]
         if 'message' in self._question and self._question['message']:
             vsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['message'],
                     dont_extend_width=True,
+                    dont_extend_height=False,
                     style='class:input.question',
                 ),
             )
         hsplit_components = [VSplit(vsplit_components, padding=1)]
         if 'description' in self._question and self._question['description']:
             hsplit_components.insert(
-                0, 
+                0,
                 Label(
                     self._question['description'],
                     style='class:input.question',
@@ -564,22 +354,23 @@ class DateRangeHandler(QHandler):
             )
         return HSplit(hsplit_components, padding=1)
 
-
-    def get_keybindings(self):
-        bindings = KeyBindings()
-
-        @bindings.add(Keys.ControlC)
-        def _ctrl_c(event):
-            get_app().exit(result=False)
-
-        @bindings.add(Keys.Enter)
-        def _enter(event):
-            get_app().exit(result=True)
-
-        return bindings
-
     def get_value(self):
         return self.get_widget().value
 
+    def get_formatted_value(self):
+        value = self.get_value()
+        return f'{value["from"]} - {value["to"]}'
 
-register('daterange', DateRangeHandler)
\ No newline at end of file
+    def get_keybindings(self):
+
+        bindings = super().get_keybindings()
+
+        @bindings.add(Keys.Tab)
+        def _tab(event):
+            get_app().layout.focus_next()
+
+        @bindings.add(Keys.BackTab)
+        def _s_tab(event):
+            get_app().layout.focus_previous()
+
+        return bindings
